import sqlite3

from .config import DATABASE


class Engine:
    def __init__(self):
        self.con = sqlite3.connect(DATABASE)
        self.cur = self.con.cursor()
        self.fields = ('id', 'domain', 'username', 'password')
        self.create()

    def create(self):
        sql = """
        create table if not exists pwd(
            id integer primary key autoincrement,
            domain text not null,
            username text not null,
            password text not null
        );
        """
        self.cur.execute(sql)
        sql = 'create unique index if not exists domain_username_index on pwd (domain, username);'
        self.cur.execute(sql)
        self.con.commit()

    def insert(self, domain: str, username: str, password: str):
        """
        :param domain: 域名
        :param username: 用户名
        :param password: 密码
        """
        sql = 'insert into pwd values (null, ?, ?, ?);'
        self.cur.execute(sql, (domain, username, password))

    def delete(self, id_: int):
        """
        :param id_: 主键ID
        """
        sql = 'delete from pwd where id = ?;'
        self.cur.execute(sql, (id_,))

    def update(self, id_: int, **kwargs):
        """
        :param id_: 主键ID
        :param kwargs: 更改的字段
        :keyword domain: 域名
        :keyword username: 用户名
        :keyword password: 密码
        """
        cond = ','.join([f'{k}=:{k}' for k in kwargs.keys()])
        sql = f'update pwd set {cond} where id=:id;'
        kwargs['id'] = id_
        self.cur.execute(sql, kwargs)

    def get(self, id_: int) -> tuple:
        """
        :param id_: 主键ID
        :return: 唯一匹配的行
        """
        sql = 'select * from pwd where id = ? limit 1;'
        self.cur.execute(sql, (id_,))
        return self.cur.fetchone()

    def filter(self, **kwargs):
        """
        :param kwargs: 条件
        :keyword domain: 域名
        :keyword username: 用户名
        :keyword password: 密码
        :return: 多个匹配的行
        :rtype: list[tuple]

        精准匹配
        """
        cond = ' and '.join([f'{k} = :{k}' for k in kwargs.keys()])
        sql = f'select * from pwd where {cond or 1};'
        self.cur.execute(sql, kwargs)
        return self.cur.fetchall()

    def like(self, **kwargs):
        """
        :param kwargs: 条件
        :keyword domain: 域名
        :keyword username: 用户名
        :keyword password: 密码
        :return: 多个匹配的行
        :rtype: list[tuple]

        模糊匹配
        """
        cond = ' or '.join([f'{k} like "%{v}%"' for k, v in kwargs.items()])
        sql = f'select * from pwd where {cond or 1};'
        self.cur.execute(sql)
        return self.cur.fetchall()

    def __del__(self):
        self.con.commit()
        self.cur.close()
        self.con.close()
